@@ -24,6 +24,7 @@
 #include <starpu_mpi_insert_task.h>
 #include <common/config.h>
 #include <common/thread.h>
+#include <datawizard/coherency.h>
 
 static void _starpu_mpi_submit_new_mpi_request(void *arg);
 static void _starpu_mpi_handle_request_termination(struct _starpu_mpi_req *req);
@@ -116,8 +117,6 @@ static void _starpu_mpi_isend_data_func(struct _starpu_mpi_req *req)
 {
 	_STARPU_MPI_LOG_IN();
 
-	STARPU_ASSERT_MSG(req->ptr, "Pointer containing data to send is invalid");
-
 	_STARPU_MPI_DEBUG(2, "post MPI isend request %p type %s tag %d src %d data %p ptr %p datatype '%s' count %d user_datatype %d \n", req, _starpu_mpi_request_type(req->request_type), req->mpi_tag, req->srcdst, req->data_handle, req->ptr, _starpu_mpi_datatype(req->datatype), (int)req->count, req->user_datatype);
 
 	_starpu_mpi_comm_amounts_inc(req->comm, req->srcdst, req->datatype, req->count);
@@ -242,8 +241,6 @@ static void _starpu_mpi_irecv_data_func(struct _starpu_mpi_req *req)
 {
 	_STARPU_MPI_LOG_IN();
 
-	STARPU_ASSERT_MSG(req->ptr, "Invalid pointer to receive data");
-
 	_STARPU_MPI_DEBUG(2, "post MPI irecv request %p type %s tag %d src %d data %p ptr %p datatype '%s' count %d user_datatype %d \n", req, _starpu_mpi_request_type(req->request_type), req->mpi_tag, req->srcdst, req->data_handle, req->ptr, _starpu_mpi_datatype(req->datatype), (int)req->count, req->user_datatype);
 
 	TRACE_MPI_IRECV_SUBMIT_BEGIN(req->srcdst, req->mpi_tag);
@@ -1034,3 +1031,16 @@ int starpu_mpi_shutdown(void)
 
 	return 0;
 }
+
+void _starpu_mpi_clear_cache(starpu_data_handle_t data_handle)
+{
+	starpu_mpi_cache_flush(MPI_COMM_WORLD, data_handle);
+}
+
+void starpu_mpi_data_register(starpu_data_handle_t data_handle, int tag, int rank)
+{
+	_starpu_data_set_rank(data_handle, rank);
+	_starpu_data_set_tag(data_handle, tag);
+	_starpu_data_set_unregister_hook(data_handle, _starpu_mpi_clear_cache);
+
+}
