@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2013  Université de Bordeaux 1
+ * Copyright (C) 2009-2014  Université de Bordeaux 1
  * Copyright (C) 2010, 2011, 2012, 2013  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ struct user_interaction_wrapper
 {
 	starpu_data_handle_t handle;
 	enum starpu_data_access_mode mode;
-	unsigned node;
+	int node;
 	starpu_pthread_cond_t cond;
 	starpu_pthread_mutex_t lock;
 	unsigned finished;
@@ -92,11 +92,16 @@ static void _starpu_data_acquire_continuation_non_blocking(void *arg)
 
 	STARPU_ASSERT(handle);
 
-	struct _starpu_data_replicate *replicate = &handle->per_node[wrapper->node];
+	if (wrapper->node >= 0)
+	{
+		struct _starpu_data_replicate *replicate = &handle->per_node[wrapper->node];
 
-	ret = _starpu_fetch_data_on_node(handle, replicate, wrapper->mode, 0, 1,
-					 _starpu_data_acquire_fetch_data_callback, wrapper);
-	STARPU_ASSERT(!ret);
+		ret = _starpu_fetch_data_on_node(handle, replicate, wrapper->mode, 0, 1,
+						 _starpu_data_acquire_fetch_data_callback, wrapper);
+		STARPU_ASSERT(!ret);
+	}
+	else
+		_starpu_data_acquire_fetch_data_callback(wrapper);
 }
 
 static void starpu_data_acquire_cb_pre_sync_callback(void *arg)
@@ -115,7 +120,7 @@ static void starpu_data_acquire_cb_pre_sync_callback(void *arg)
 }
 
 /* The data must be released by calling starpu_data_release later on */
-int starpu_data_acquire_on_node_cb(starpu_data_handle_t handle, unsigned node,
+int starpu_data_acquire_on_node_cb(starpu_data_handle_t handle, int node,
 			   enum starpu_data_access_mode mode, void (*callback)(void *), void *arg)
 {
 	STARPU_ASSERT(handle);
@@ -197,9 +202,14 @@ static inline void _starpu_data_acquire_continuation(void *arg)
 
 	STARPU_ASSERT(handle);
 
-	struct _starpu_data_replicate *replicate = &handle->per_node[wrapper->node];
+	if (wrapper->node >= 0)
+	{
+		int ret;
+		struct _starpu_data_replicate *replicate = &handle->per_node[wrapper->node];
 
-	_starpu_fetch_data_on_node(handle, replicate, wrapper->mode, 0, 0, NULL, NULL);
+		ret = _starpu_fetch_data_on_node(handle, replicate, wrapper->mode, 0, 0, NULL, NULL);
+		STARPU_ASSERT(!ret);
+	}
 
 	/* continuation of starpu_data_acquire */
 	STARPU_PTHREAD_MUTEX_LOCK(&wrapper->lock);
@@ -209,7 +219,7 @@ static inline void _starpu_data_acquire_continuation(void *arg)
 }
 
 /* The data must be released by calling starpu_data_release later on */
-int starpu_data_acquire_on_node(starpu_data_handle_t handle, unsigned node, enum starpu_data_access_mode mode)
+int starpu_data_acquire_on_node(starpu_data_handle_t handle, int node, enum starpu_data_access_mode mode)
 {
 	STARPU_ASSERT(handle);
 	STARPU_ASSERT_MSG(handle->nchildren == 0, "Acquiring a partitioned data is not possible");
@@ -286,10 +296,13 @@ int starpu_data_acquire_on_node(starpu_data_handle_t handle, unsigned node, enum
  	* available again, otherwise we fetch the data directly */
 	if (!_starpu_attempt_to_submit_data_request_from_apps(handle, mode, _starpu_data_acquire_continuation, &wrapper))
 	{
-		/* no one has locked this data yet, so we proceed immediately */
-		struct _starpu_data_replicate *replicate = &handle->per_node[node];
-		int ret = _starpu_fetch_data_on_node(handle, replicate, mode, 0, 0, NULL, NULL);
-		STARPU_ASSERT(!ret);
+		if (node >= 0)
+		{
+			/* no one has locked this data yet, so we proceed immediately */
+			struct _starpu_data_replicate *replicate = &handle->per_node[node];
+			int ret = _starpu_fetch_data_on_node(handle, replicate, mode, 0, 0, NULL, NULL);
+			STARPU_ASSERT(!ret);
+		}
 	}
 	else
 	{
@@ -318,7 +331,7 @@ int starpu_data_acquire(starpu_data_handle_t handle, enum starpu_data_access_mod
 
 /* This function must be called after starpu_data_acquire so that the
  * application release the data */
-void starpu_data_release_on_node(starpu_data_handle_t handle, unsigned node)
+void starpu_data_release_on_node(starpu_data_handle_t handle, int node)
 {
 	STARPU_ASSERT(handle);
 
@@ -326,7 +339,14 @@ void starpu_data_release_on_node(starpu_data_handle_t handle, unsigned node)
 	_starpu_unlock_post_sync_tasks(handle);
 
 	/* The application can now release the rw-lock */
-	_starpu_release_data_on_node(handle, 0, &handle->per_node[node]);
+	if (node >= 0)
+		_starpu_release_data_on_node(handle, 0, &handle->per_node[node]);
+	else
+	{
+		_starpu_spin_lock(&handle->header_lock);
+		if (!_starpu_notify_data_dependencies(handle))
+			_starpu_spin_unlock(&handle->header_lock);
+	}
 }
 
 void starpu_data_release(starpu_data_handle_t handle)
@@ -365,8 +385,7 @@ int _starpu_prefetch_data_on_node_with_mode(starpu_data_handle_t handle, unsigne
 	STARPU_ASSERT(handle);
 
 	/* it is forbidden to call this function from a callback or a codelet */
-	if (STARPU_UNLIKELY(!async && !_starpu_worker_may_perform_blocking_calls()))
-		return -EDEADLK;
+	STARPU_ASSERT_MSG(async || _starpu_worker_may_perform_blocking_calls(), "Synchronous prefetch is not possible from a task or a callback");
 
 	struct user_interaction_wrapper *wrapper = (struct user_interaction_wrapper *) malloc(sizeof(*wrapper));
 
@@ -491,9 +510,7 @@ void starpu_data_set_default_sequential_consistency_flag(unsigned flag)
 /* Query the status of the handle on the specified memory node. */
 void starpu_data_query_status(starpu_data_handle_t handle, int memory_node, int *is_allocated, int *is_valid, int *is_requested)
 {
-#ifdef STARPU_DEVEL
-#warning FIXME
-#endif
+// XXX : this is just a hint, so we don't take the lock ...
 //	_starpu_spin_lock(&handle->header_lock);
 
 	if (is_allocated)
@@ -509,7 +526,7 @@ void starpu_data_query_status(starpu_data_handle_t handle, int memory_node, int
 		unsigned node;
 		for (node = 0; node < STARPU_MAXNODES; node++)
 		{
-			if (handle->per_node[memory_node].requested[node])
+			if (handle->per_node[memory_node].requested & (1UL << node))
 			{
 				requested = 1;
 				break;
